@@ -12,6 +12,8 @@
 void AddIncludeExclude(HashratCtx *Ctx, int Type, const char *Item)
 {
 ListNode *Node;
+char *Token=NULL;
+const char *ptr;
 
 //if we get given an include with no previous excludes, then 
 //set CTX_EXCLUDE as the default
@@ -21,8 +23,15 @@ if (! IncludeExclude)
   if (Type==CTX_INCLUDE) Ctx->Flags |= CTX_EXCLUDE;
 }
 
-Node=ListAddItem(IncludeExclude, CopyStr(NULL, Item));
-Node->ItemType=Type;
+ptr=GetToken(Item, ",",&Token, GETTOKEN_QUOTES);
+while (ptr)
+{
+	Node=ListAddItem(IncludeExclude, CopyStr(NULL, Token));
+	Node->ItemType=Type;
+	ptr=GetToken(ptr, ",",&Token, GETTOKEN_QUOTES);
+}
+
+DestroyString(Token);
 }
 
 
@@ -93,22 +102,15 @@ return(ParseFlags);
 void CommandLineSetCtx(int argc, char *argv[], int pos, HashratCtx *Ctx, int Flag, int Encoding)
 {
 if (Encoding > 0) Ctx->Encoding=Encoding;
-Ctx->Flags |= Flag;
 strcpy(argv[pos],"");
 
-if (Flag == CTX_INCLUDE) 
+if ((Flag==CTX_INCLUDE) || (Flag==CTX_EXCLUDE))
 {
 	pos++;
-	AddIncludeExclude(Ctx,CTX_INCLUDE, argv[pos]);
+	AddIncludeExclude(Ctx, Flag, argv[pos]);
 	strcpy(argv[pos],"");
 }
-else if (Flag == CTX_EXCLUDE) 
-{
-	pos++;
-	AddIncludeExclude(Ctx,CTX_EXCLUDE, argv[pos]);
-	strcpy(argv[pos],"");
-}
-
+else Ctx->Flags |= Flag;
 }
 
 
@@ -129,7 +131,7 @@ while (ptr)
 	else if (strcasecmp(Token, "txattr")==0) Ctx->Flags |= CTX_STORE_XATTR | CTX_XATTR_ROOT;
 	else if (strcasecmp(Token, "memcached")==0) Ctx->Flags |= CTX_STORE_MEMCACHED;
 	else if (strcasecmp(Token, "mcd")==0) Ctx->Flags |= CTX_STORE_MEMCACHED;
-	else if (! Ctx->Aux) Ctx->Aux=STREAMOpenFile(Token,STREAM_WRONLY | STREAM_CREAT | STREAM_TRUNC);
+	else if (! Ctx->Aux) Ctx->Aux=STREAMOpenFile(Token,SF_WRONLY | SF_CREAT | SF_TRUNC);
 
 ptr=GetToken(ptr,",",&Token,0);
 }
@@ -286,6 +288,12 @@ else if (strcmp(argv[i],"-cB")==0)
 	Ctx->Action = ACT_CHECKBACKUP;
 	strcpy(argv[i],"");
 }
+else if ((strcmp(argv[i],"-dir")==0) || (strcmp(argv[i],"-dirmode")==0))
+{
+	Ctx->Action = ACT_HASHDIR;
+	Ctx->Flags |= CTX_RECURSE;
+	strcpy(argv[i],"");
+}
 else if ((strcmp(argv[i],"-hook")==0) || (strcmp(argv[i],"-h")==0))
 {
 	strcpy(argv[i],"");
@@ -322,6 +330,12 @@ else if (strcmp(argv[i],"-jh256")==0) ParseFlags |= CommandLineHandleArg(argc, a
 else if (strcmp(argv[i],"-jh384")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, 0, "HashType", "jh-384",Ctx->Vars);
 else if (strcmp(argv[i],"-jh512")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, 0, "HashType", "jh-512",Ctx->Vars);
 else if (strcmp(argv[i],"-jh")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, 0, "HashType", "jh-512",Ctx->Vars);
+else if (strcmp(argv[i],"-type")==0) 
+{
+	strcpy(argv[i],"");
+	i++;
+	ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, 0, "HashType", argv[i],Ctx->Vars);
+}
 //else if (strcmp(argv[i],"-crc32")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, 0, "HashType", "crc32",Ctx->Vars);
 else if (strcmp(argv[i],"-8")==0)  CommandLineSetCtx(argc, argv, i, Ctx,  0, ENCODE_OCTAL);
 else if (strcmp(argv[i],"-10")==0) CommandLineSetCtx(argc, argv, i, Ctx,  0, ENCODE_DECIMAL);
@@ -348,8 +362,7 @@ else if (strcmp(argv[i],"-idfile")==0) ParseFlags |= CommandLineHandleArg(argc,
 else if (strcmp(argv[i],"-f")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, CMDLINE_FROM_LISTFILE, 0, "", "",Ctx->Vars);
 else if (strcmp(argv[i],"-i")==0) CommandLineSetCtx(argc, argv, i, Ctx, CTX_INCLUDE,0);
 else if (strcmp(argv[i],"-x")==0) CommandLineSetCtx(argc, argv, i, Ctx, CTX_EXCLUDE,0);
-else if (strcmp(argv[i],"-dirmode")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, FLAG_DIRMODE, "", "",Ctx->Vars);
-else if (strcmp(argv[i],"-devmode")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, FLAG_DIRMODE | FLAG_DEVMODE, "", "",Ctx->Vars);
+else if (strcmp(argv[i],"-devmode")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, FLAG_DEVMODE, "", "",Ctx->Vars);
 else if (strcmp(argv[i],"-lines")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, FLAG_LINEMODE, "", "",Ctx->Vars);
 else if (strcmp(argv[i],"-rawlines")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, FLAG_RAW|FLAG_LINEMODE, "", "",Ctx->Vars);
 else if (strcmp(argv[i],"-hide-input")==0) ParseFlags |= CommandLineHandleArg(argc, argv, i, 0, FLAG_HIDE_INPUT, "", "",Ctx->Vars);
@@ -513,8 +526,8 @@ printf("  %-15s %s\n","-tag", "Output hashes in bsdsum format");
 printf("  %-15s %s\n","--tag", "Output hashes in bsdsum format");
 printf("  %-15s %s\n","-r", "Recurse into directories when hashing files");
 printf("  %-15s %s\n","-f <listfile>", "Hash files listed in <listfile>");
-printf("  %-15s %s\n","-i <pattern>", "Only hash items matching <pattern>");
-printf("  %-15s %s\n","-x <pattern>", "Exclude items matching <pattern>");
+printf("  %-15s %s\n","-i <patterns>", "Only hash items matching a comma-seperated list of shell patterns");
+printf("  %-15s %s\n","-x <patterns>", "Exclude items matching a comma-sepearted list of shell patterns");
 printf("  %-15s %s\n","-n <length>", "Truncate hashes to <length> bytes");
 printf("  %-15s %s\n","-c", "CHECK hashes against list from file (or stdin)");
 printf("  %-15s %s\n","-cf", "CHECK hashes against list but only show failures");
@@ -534,6 +547,7 @@ printf("  %-15s %s\n","-strict", "Strict mode: when checking, check file mtime,
 printf("  %-15s %s\n","-S", "Strict mode: when checking, check file mtime, owner, group, and inode as well as it's hash");
 printf("  %-15s %s\n","-d","dereference (follow) symlinks"); 
 printf("  %-15s %s\n","-fs", "Stay on one file system");
+printf("  %-15s %s\n","-dir", "DirMode: Read all files in directory and create one hash for them!");
 printf("  %-15s %s\n","-dirmode", "DirMode: Read all files in directory and create one hash for them!");
 printf("  %-15s %s\n","-devmode", "DevMode: read from a file EVEN OF IT'S A DEVNODE");
 printf("  %-15s %s\n","-lines", "Read lines from stdin and hash each line independantly.");
