@@ -6,6 +6,8 @@
 #include "list.h"
 
 
+#define STREAM_FLUSH -1
+
 #define STREAM_CLOSED -1
 #define STREAM_NODATA -2
 #define STREAM_TIMEOUT -3
@@ -39,22 +41,24 @@
 #define FLUSH_ALWAYS 4
 #define FLUSH_BUFFER 8
 
-#define STREAM_RDWR 0 //is the default
-#define STREAM_RDONLY 16
-#define STREAM_WRONLY 32
-#define STREAM_CREAT 64
-#define STREAM_CREATE 64
-#define STREAM_APPEND 128
-#define STREAM_TRUNC 256
-#define STREAM_SYMLINK_OK 512
-#define STREAM_WRLOCK 1024
-#define STREAM_RDLOCK 2048
-#define STREAM_FOLLOW 4096
-#define STREAM_SECURE 8192
-#define STREAM_NONBLOCK 16384
-#define STREAM_SSL 32768
-#define STREAM_AUTH 65536
-
+#define SF_RDWR 0 //is the default
+//FLUSH_ flags go in this gap
+#define SF_RDONLY 16
+#define SF_WRONLY 32
+#define SF_CREAT 64
+#define SF_CREATE 64
+#define SF_APPEND 128
+#define SF_TRUNC 256
+#define SF_MMAP  512
+#define SF_WRLOCK 1024
+#define SF_RDLOCK 2048
+#define SF_FOLLOW 4096
+#define SF_SECURE 8192
+#define SF_NONBLOCK 16384
+#define SF_EXEC_INHERIT 131072
+#define SF_SYMLINK_OK 262144
+#define SF_NOCACHE 524288
+#define SF_SORTED  1048576
 
 
 //Stream state values
@@ -64,6 +68,7 @@
 #define SS_DATA_ERROR 8
 #define SS_WRITE_ERROR 16
 #define SS_EMBARGOED 32
+#define SS_SSL  4096
 #define SS_AUTH 8192
 #define SS_USER1 268435456
 #define SS_USER2 536870912
@@ -78,6 +83,7 @@
 #define STREAM_TYPE_SSL 5
 #define STREAM_TYPE_HTTP 6
 #define STREAM_TYPE_CHUNKED_HTTP 7
+#define STREAM_TYPE_MESSAGEBUS 8
 
 #define O_LOCK O_NOCTTY
 
@@ -85,6 +91,10 @@
 #define SELECT_READ 1
 #define SELECT_WRITE 2	
 
+
+#define SENDFILE_KERNEL 1
+#define SENDFILE_LOOP 2
+
 typedef struct
 {
 int Type;
@@ -143,7 +153,6 @@ int STREAMWriteString(const char *Buffer, STREAM *);
 int STREAMWriteLine(const char *Buffer, STREAM *);
 STREAM *STREAMFromFD(int fd);
 STREAM *STREAMFromDualFD(int in_fd, int out_fd);
-STREAM *STREAMSpawnCommand(const char *Command, int type);
 
 int STREAMDisassociateFromFD(STREAM *Stream);
 int STREAMPeekChar(STREAM *);
@@ -163,7 +172,7 @@ char *STREAMGetValue(STREAM *S, const char *Name);
 void STREAMSetItem(STREAM *S, const char *Name, void *Item);
 void *STREAMGetItem(STREAM *S, const char *Name);
 
-int STREAMSendFile(STREAM *In, STREAM *Out, off_t Max);
+unsigned long STREAMSendFile(STREAM *In, STREAM *Out, unsigned long Max, int Flags);
 
 #ifdef __cplusplus
 }
