@@ -1,5 +1,7 @@
 #include "socket.h"
 #include "ConnectionChain.h"
+#include "ParseURL.h"
+#include "unix_socket.h"
 
 #include <arpa/inet.h>
 #include <net/if.h>
@@ -87,6 +89,137 @@ return(FALSE);
 }
 
 
+//Socket options wrapped in ifdef statements to handle systems that lack certain options
+int SockSetOptions(int sock, int SetFlags, int UnsetFlags)
+{
+int result;
+
+result=TRUE;
+#ifdef SO_BROADCAST
+if (SetFlags & SOCK_BROADCAST) result=setsockopt(sock, SOL_SOCKET, SO_BROADCAST, &result,sizeof(int));
+#endif
+
+#ifdef SO_DONTROUTE
+if (SetFlags & SOCK_DONTROUTE) result=setsockopt(sock, SOL_SOCKET, SO_DONTROUTE, &result,sizeof(int));
+#endif
+
+#ifdef SO_REUSEPORT
+if (SetFlags & SOCK_REUSEPORT) result=setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &result,sizeof(int));
+#endif
+
+#ifdef IP_TRANSPARENT
+if (SetFlags & SOCK_TPROXY) result=setsockopt(sock, IPPROTO_IP, IP_TRANSPARENT, &result,sizeof(int));
+#endif
+
+#ifdef SO_PASSCRED
+if (SetFlags & SOCK_PEERCREDS) result=setsockopt(sock, SOL_SOCKET, SO_PASSCRED, &result,sizeof(int));
+#endif
+
+#ifdef SO_KEEPALIVE
+//Default is KEEPALIVE ON
+setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, &result,sizeof(int));
+#endif
+
+if (SetFlags & CONNECT_NONBLOCK) fcntl(sock,F_SETFL,O_NONBLOCK);
+
+
+#ifdef SO_BROADCAST
+if (UnsetFlags & SOCK_BROADCAST) result=setsockopt(sock, SOL_SOCKET, SO_BROADCAST, &result,sizeof(int));
+#endif
+
+#ifdef SO_DONTROUTE
+if (UnsetFlags & SOCK_DONTROUTE) result=setsockopt(sock, SOL_SOCKET, SO_DONTROUTE, &result,sizeof(int));
+#endif
+
+#ifdef SO_REUSEPORT
+if (UnsetFlags & SOCK_REUSEPORT) result=setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &result,sizeof(int));
+#endif
+
+#ifdef IP_TRANSPARENT
+if (UnsetFlags & SOCK_TPROXY) result=setsockopt(sock, IPPROTO_IP, IP_TRANSPARENT, &result,sizeof(int));
+#endif
+
+#ifdef SO_PASSCRED
+if (UnsetFlags & SOCK_PEERCREDS) result=setsockopt(sock, SOL_SOCKET, SO_PASSCRED, &result,sizeof(int));
+#endif
+
+result=FALSE;
+#ifdef SO_KEEPALIVE
+if (SetFlags & SOCK_NOKEEPALIVE) setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, &result,sizeof(int));
+#endif
+}
+
+
+
+
+
+int SockSetAttribute(int sock, int Attrib, int Value)
+{
+
+
+}
+
+int SockGetAttribute(int sock, int Attrib, int Value)
+{
+
+
+}
+
+
+
+int GetHostARP(const char *IP, char **Device, char **MAC)
+{
+char *Tempstr=NULL, *Token=NULL;
+int result=FALSE;
+const char *ptr;
+FILE *F;
+
+Tempstr=SetStrLen(Tempstr, 255);
+F=fopen("/proc/net/arp","r");
+if (F)
+{
+	*Device=CopyStr(*Device,"remote");
+	*MAC=CopyStr(*MAC,"remote");
+	//Read Title Line
+	fgets(Tempstr,255,F);
+
+	while (fgets(Tempstr,255,F))
+	{
+		StripTrailingWhitespace(Tempstr);
+		ptr=GetToken(Tempstr," ",&Token,0);
+		if (strcmp(Token,IP)==0)
+		{
+			while (isspace(*ptr)) ptr++;
+			ptr=GetToken(ptr," ",&Token,0);
+
+			while (isspace(*ptr)) ptr++;
+			ptr=GetToken(ptr," ",&Token,0);
+
+			while (isspace(*ptr)) ptr++;
+			ptr=GetToken(ptr," ",MAC,0);
+			strlwr(*MAC);
+
+			while (isspace(*ptr)) ptr++;
+			ptr=GetToken(ptr," ",&Token,0);
+
+			while (isspace(*ptr)) ptr++;
+			ptr=GetToken(ptr," ",Device,0);
+
+			result=TRUE;
+		}
+	}
+fclose(F);
+}
+
+DestroyString(Tempstr);
+DestroyString(Token);
+
+return(result);
+}
+
+
+
+
 const char *GetInterfaceIP(const char *Interface)
 {
  int fd, result;
@@ -105,68 +238,186 @@ const char *GetInterfaceIP(const char *Interface)
 }
 
 
+
+#include <netinet/ip_icmp.h>
+
+/*--------------------------------------------------------------------*/
+/*--- checksum - standard 1s complement checksum                   ---*/
+/*--------------------------------------------------------------------*/
+unsigned short checksum(void *b, int len)
+{
+    unsigned short *buf = b;
+    unsigned int sum=0;
+    unsigned short result;
+
+    for ( sum = 0; len > 1; len -= 2 )
+        sum += *buf++;
+    if ( len == 1 )
+        sum += *(unsigned char*)buf;
+    sum = (sum >> 16) + (sum & 0xFFFF);
+    sum += (sum >> 16);
+    result = ~sum;
+    return result;
+}
+
+
+int ICMPSend(int sock, const char *Host, int Type, int Code, int TTL, char *Data, int len)
+{
+struct icmphdr *ICMPHead;
+char *Tempstr=NULL;
+int packet_len;
+static int seq=0;
+struct sockaddr_in sa;
+int result;
+
+result=TTL;
+if (setsockopt(sock, SOL_IP, IP_TTL, &result, sizeof(int)) != 0) return(0);
+
+packet_len=len+sizeof(struct icmphdr);
+Tempstr=(char *) calloc(1, packet_len+1);
+ICMPHead=(struct icmphdr *) Tempstr;
+ICMPHead->type=ICMP_ECHO;
+ICMPHead->un.echo.id=getpid();
+ICMPHead->un.echo.sequence=seq++;
+if ((len > 0) && Data) memcpy(Tempstr + sizeof(struct icmphdr), Data, len);
+ICMPHead->checksum=checksum(Tempstr, packet_len);
+
+memset(&sa, 0, sizeof(sa));
+sa.sin_family=AF_INET;
+sa.sin_port=0;
+sa.sin_addr.s_addr=StrtoIP(Host);
+result=sendto(sock, Tempstr, packet_len, 0, (struct sockaddr*) &sa, sizeof(struct sockaddr));
+
+DestroyString(Tempstr);
+
+return(result);
+}
+
+
+int SockAddrCreate(struct sockaddr **ret_sa, int Family, const char *Addr, int Port)
+{
+struct sockaddr_in *sa4;
+struct sockaddr_in6 *sa6;
+char *Token=NULL, *ptr;
+socklen_t salen;
+
+if (Family==AF_INET)
+{
+	sa4=(struct sockaddr_in *) calloc(1,sizeof(struct sockaddr_in));
+	if (StrLen(Addr)) sa4->sin_addr.s_addr=StrtoIP(Addr);
+	else sa4->sin_addr.s_addr=INADDR_ANY;
+	sa4->sin_port=htons(Port);
+	sa4->sin_family=AF_INET;
+	*ret_sa=(struct sockaddr *) sa4;
+	salen=sizeof(struct sockaddr_in);
+}
+else
+{
+	sa6=(struct sockaddr_in6 *) calloc(1,sizeof(struct sockaddr_in6));
+	if (StrLen(Addr)) 
+	{
+		ptr=GetToken(Addr, "%",&Token,0);
+		if (StrLen(ptr)) sa6->sin6_scope_id=if_nametoindex(ptr);
+		inet_pton(AF_INET6, Token, &(sa6->sin6_addr));
+	}
+	else sa6->sin6_addr=in6addr_any;
+	sa6->sin6_port=htons(Port);
+	sa6->sin6_family=AF_INET6;
+
+	*ret_sa=(struct sockaddr *) sa6;
+	salen=sizeof(struct sockaddr_in6);
+}
+
+DestroyString(Token);
+
+return(salen);
+}
+
+
+int UDPOpen(const char *Addr, int Port, int Flags)
+{
+	int result, Family=AF_INET6;
+	struct sockaddr *sa;
+	socklen_t salen;
+	int fd;
+
+	if (IsIP4Address(Addr)) Family=AF_INET; 
+	salen=SockAddrCreate(&sa, Family, Addr, Port);
+	fd=socket(Family, SOCK_DGRAM,0);
+	result=bind(fd, sa, salen);
+	if (result !=0)
+	{
+		close(fd);
+		return(-1);
+	}
+
+	SockSetOptions(fd, Flags, 0);
+	return(fd);
+}
+
+
+
+
+int UDPSend(int sock, const char *Host, int Port, char *Data, int len)
+{
+int Family=AF_INET;
+struct sockaddr *sa=NULL;
+socklen_t salen;
+int result;
+
+if (IsIP6Address(Host)) Family=AF_INET6; 
+
+salen=SockAddrCreate(&sa, Family, Host, Port);
+
+result=sendto(sock, Data, len, 0, sa , salen);
+free(sa);
+return(result);
+}
+
+
+
+int STREAMSendDgram(STREAM *S, const char *Host, int Port, char *Data, int len)
+{
+return(UDPSend(S->out_fd, Host, Port, Data, len));
+}
+
+
 #ifdef USE_INET6
 int InitServerSock(int Type, const char *Address, int Port)
 {
 int sock;
-struct sockaddr_storage sa;
+//struct sockaddr_storage sa;
 struct sockaddr_in *sa4;
 struct sockaddr_in6 *sa6;
+struct sockaddr *sa=NULL;
 socklen_t salen;
-int result;
-char *p_Addr=NULL, *ptr;
+int result, Family=AF_INET6;
+const char *p_Addr=NULL, *ptr;
 
 
 if (StrLen(Address))  
 {
 	if (IsIP4Address(Address)) 
 	{
-		sa.ss_family=AF_INET;
+		Family=AF_INET;
 		p_Addr=Address;
 	}
 	else if (IsIP6Address(Address)) 
 	{
-		sa.ss_family=AF_INET6;
+		Family=AF_INET6;
 		p_Addr=Address;
 	}
 	else 
 	{
-		sa.ss_family=AF_INET;
+		Family=AF_INET;
 		p_Addr=GetInterfaceIP(Address);
 	}
 }
-else sa.ss_family=AF_INET;
 
-if (sa.ss_family==AF_INET)
-{
-	sa4=(struct sockaddr_in *) &sa;
-	if (StrLen(p_Addr)) sa4->sin_addr.s_addr=StrtoIP(p_Addr);
-	else sa4->sin_addr.s_addr=INADDR_ANY;
-	sa4->sin_port=htons(Port);
-	sa4->sin_family=AF_INET;
-	salen=sizeof(struct sockaddr_in);
-}
-else
-{
-	sa6=(struct sockaddr_in6 *) &sa;
-	sa6->sin6_port=htons(Port);
-	sa6->sin6_family=AF_INET6;
-	if (StrLen(p_Addr)) 
-	{
-		ptr=strchr(p_Addr,'%');
-		if (ptr)
-		{
-			sa6->sin6_scope_id=if_nametoindex(ptr+1);
-			*ptr='\0';	
-		}
-		inet_pton(AF_INET6, p_Addr, &(sa6->sin6_addr));
-	}
-	else sa6->sin6_addr=in6addr_any;
-	salen=sizeof(struct sockaddr_in6);
-}
+salen=SockAddrCreate(&sa, Family, p_Addr, Port);
 
 if (Type==0) Type=SOCK_STREAM;
-sock=socket(sa.ss_family,Type,0);
+sock=socket(Family,Type,0);
 if (sock <0) return(-1);
 
 //No reason to pass server/listen sockets across an exec
@@ -175,20 +426,20 @@ fcntl(sock, F_SETFD, FD_CLOEXEC);
 result=1;
 setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,&result,sizeof(result));
 
-result=bind(sock,(struct sockaddr *) &sa, salen);
+result=bind(sock, sa, salen);
 
 if ((result==0) && (Type==SOCK_STREAM))
 {
  result=listen(sock,10);
 }
 
+free(sa);
+
 if (result==0) return(sock);
-else 
-{
+
 close(sock);
 return(-1);
 }
-}
 
 #else 
 
@@ -244,7 +495,6 @@ return(-1);
 #endif
 
 
-
 int TCPServerSockAccept(int ServerSock, char **Addr)
 {
 struct sockaddr_storage sa;
@@ -252,8 +502,8 @@ socklen_t salen;
 int sock;
 
 salen=sizeof(sa);
-sock=accept(ServerSock,(struct sockaddr *) &sa,&salen);
-if (Addr) 
+sock=accept(ServerSock,(struct sockaddr *) &sa, &salen);
+if (Addr && (sock > -1)) 
 {
 *Addr=SetStrLen(*Addr,NI_MAXHOST);
 getnameinfo((struct sockaddr *) &sa, salen, *Addr, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
@@ -265,13 +515,18 @@ return(sock);
 
 
 
+
+
+
+
+
 #ifdef USE_INET6
 int GetSockDetails(int sock, char **LocalAddress, int *LocalPort, char **RemoteAddress, int *RemotePort)
 {
 socklen_t salen;
 int result;
 struct sockaddr_storage sa;
-char *Tempstr=NULL;
+char *Tempstr=NULL, *ptr;
 
 *LocalPort=0;
 *RemotePort=0;
@@ -283,11 +538,19 @@ result=getsockname(sock, (struct sockaddr *) &sa, &salen);
 
 if (result==0)
 {
-*LocalAddress=SetStrLen(*LocalAddress,NI_MAXHOST);
-Tempstr=SetStrLen(Tempstr,NI_MAXSERV);
-getnameinfo((struct sockaddr *) &sa, salen, *LocalAddress, NI_MAXHOST, Tempstr, NI_MAXSERV, NI_NUMERICHOST|NI_NUMERICSERV);
+	*LocalAddress=SetStrLen(*LocalAddress,NI_MAXHOST);
+	Tempstr=SetStrLen(Tempstr,NI_MAXSERV);
+	getnameinfo((struct sockaddr *) &sa, salen, *LocalAddress, NI_MAXHOST, Tempstr, NI_MAXSERV, NI_NUMERICHOST|NI_NUMERICSERV);
 	*LocalPort=atoi(Tempstr);
 
+	if ((strncmp(*LocalAddress,"::ffff:",7)==0) && strchr(*LocalAddress,'.')) 
+	{
+		ptr=*LocalAddress;
+		ptr+=7;
+		memmove(*LocalAddress,ptr,StrLen(*LocalAddress)-6);
+	}
+	
+
 	//Set Address to be the same as control sock, as it might not be INADDR_ANY
 	result=getpeername(sock, (struct sockaddr *) &sa, &salen);
 
@@ -298,6 +561,12 @@ getnameinfo((struct sockaddr *) &sa, salen, *LocalAddress, NI_MAXHOST, Tempstr,
 		getnameinfo((struct sockaddr *) &sa, salen, *RemoteAddress, NI_MAXHOST, Tempstr, NI_MAXSERV, NI_NUMERICHOST|NI_NUMERICSERV);
 		*RemotePort=atoi(Tempstr);
 
+		if ((strncmp(*RemoteAddress,"::ffff:",7)==0) && strchr(*RemoteAddress,'.')) 
+		{
+		ptr=*RemoteAddress;
+		ptr+=7;
+		memmove(*RemoteAddress,ptr,StrLen(*RemoteAddress)-6);
+		}
 	}
 
 	//We've got the local sock, so lets still call it a success
@@ -356,7 +625,7 @@ return(FALSE);
 /* Users will probably only use this function if they want to reconnect   */
 /* a broken connection, or reuse a socket for multiple connections, hence */
 /* the name... */
-int ReconnectSock(int sock, const char *Host, int Port, int Flags)
+int IPReconnect(int sock, const char *Host, int Port, int Flags)
 {
 socklen_t salen;
 int result;
@@ -381,10 +650,7 @@ sa.sin_addr=*(struct in_addr *) *hostdata->h_addr_list;
 }
 
 salen=sizeof(sa);
-if (Flags & CONNECT_NONBLOCK) 
-{
-fcntl(sock,F_SETFL,O_NONBLOCK);
-}
+SockSetOptions(sock, Flags, 0);
 
 result=connect(sock,(struct sockaddr *)&sa, salen);
 if (result==0) result=TRUE;
@@ -394,13 +660,17 @@ return(result);
 }
 
 
-int ConnectToHost(const char *Host, int Port,int Flags)
+int TCPConnectWithAttributes(const char *Host, int Port, int Flags, int TTL, int ToS)
 {
 int sock, result;
 
 sock=socket(AF_INET,SOCK_STREAM,0);
 if (sock <0) return(-1);
-result=ReconnectSock(sock,Host,Port,Flags);
+
+if (TTL > 0) setsockopt(sock, IPPROTO_IP, IP_TTL, &TTL, sizeof(int));
+if (ToS > 0) setsockopt(sock, IPPROTO_IP, IP_TOS, &ToS, sizeof(int));
+
+result=IPReconnect(sock,Host,Port,Flags);
 if (result==-1)
 {
 close(sock);
@@ -412,6 +682,11 @@ return(sock);
 }
 
 
+int TCPConnect(const char *Host, int Port, int Flags)
+{
+return(TCPConnectWithAttributes(Host, Port, Flags, 0, 0));
+}
+
 
 
 int CheckForTerm(DownloadContext *CTX, char inchar)
@@ -604,15 +879,17 @@ if ((S->in_fd > -1) && (S->Timeout > 0) )
     S->in_fd=-1;
     S->out_fd=-1;
   }
-  else if (! (Flags & CONNECT_NONBLOCK))  STREAMSetFlags(S, 0, STREAM_NONBLOCK);
 }
 
 if (S->in_fd > -1)
 {
+//We will have been non-blocking during connection, but if we don't 
+//really want the stream to be non blocking, we unset that here
+if (! (Flags & CONNECT_NONBLOCK))  STREAMSetFlags(S, 0, SF_NONBLOCK);
+
 S->Type=STREAM_TYPE_TCP;
 result=TRUE;
 STREAMSetFlushType(S,FLUSH_LINE,0,0);
-//if (Flags & CONNECT_SOCKS_PROXY) result=DoSocksProxyTunnel(S);
 if (Flags & CONNECT_SSL) DoSSLClientNegotiation(S, Flags);
 
 ptr=GetRemoteIP(S->in_fd);
@@ -622,50 +899,52 @@ if (ptr) STREAMSetValue(S,"PeerIP",ptr);
 return(result);
 }
 
+extern char *GlobalConnectionChain;
 
-
-int STREAMConnectToHost(STREAM *S, const char *DesiredHost, int DesiredPort,int Flags)
+int STREAMTCPConnect(STREAM *S, const char *Host, int Port, int TTL, int ToS, int Flags)
 {
-ListNode *Curr;
-char *Token=NULL;
-int result=FALSE;
-int HopNo=0, val=0;
-ListNode *LastHop=NULL;
+ListNode *Curr, *LastHop;
+char *Token=NULL, *ptr;
+int HopNo=0, result=FALSE;
+
+
+if (! STREAMGetValue(S, "ConnectHop:0"))
+{
+if (StrValid(GlobalConnectionChain))  STREAMAddConnectionHop(S, GlobalConnectionChain);
+}
 
-S->Path=FormatStr(S->Path,"tcp:%s:%d",DesiredHost,DesiredPort);
 //Find the last hop, used to decide what ssh command to use
 Curr=ListGetNext(S->Values);
 while (Curr)
 {
 GetToken(Curr->Tag,":",&Token,0);
 if (strcasecmp(Token,"ConnectHop")==0) LastHop=Curr;
+if (strcasecmp(Curr->Tag,"TTL")==0) TTL=atoi(Curr->Item);
+if (strcasecmp(Curr->Tag,"ToS")==0) ToS=atoi(Curr->Item);
 Curr=ListGetNext(Curr);
 }
 
-STREAMSetFlushType(S,FLUSH_LINE,0,0);
+
 Curr=ListGetNext(S->Values);
 while (Curr)
 {
-GetToken(Curr->Tag,":",&Token,0);
+	GetToken(Curr->Tag,":",&Token,0);
 
-if (strcasecmp(Token,"ConnectHop")==0) result=STREAMProcessConnectHop(S, (char *) Curr->Item,Curr==LastHop);
+	if (strcasecmp(Token,"ConnectHop")==0) result=STREAMProcessConnectHop(S, (char *) Curr->Item,Curr==LastHop);
 
-HopNo++;
-if (! result) break;
-Curr=ListGetNext(Curr);
+	HopNo++;
+	if (! result) break;
+	Curr=ListGetNext(Curr);
 }
 
 //If we're not handling the connection through 'Connect hops' then
 //just connect to host
-if ((HopNo==0) && StrLen(DesiredHost))
+if ((HopNo==0) && StrLen(Host))
 {
-	if (Flags & CONNECT_NONBLOCK) S->Flags |= STREAM_NONBLOCK;
-	val=Flags;
-
-	//STREAMDoPostConnect handles this	
-	if (S->Timeout > 0) val |= CONNECT_NONBLOCK;
-
-	S->in_fd=ConnectToHost(DesiredHost,DesiredPort,val);
+	if (Flags & CONNECT_NONBLOCK) S->Flags |= SF_NONBLOCK;
+	
+	//Flags are handled in this function
+	S->in_fd=TCPConnectWithAttributes(Host,Port,Flags,TTL,ToS);
 
 	S->out_fd=S->in_fd;
 	if (S->in_fd > -1) result=TRUE;
@@ -676,7 +955,7 @@ if (result==TRUE)
 	if (Flags & CONNECT_NONBLOCK) 
 	{
 		S->State |=SS_CONNECTING;
-		S->Flags |=STREAM_NONBLOCK;
+		S->Flags |=SF_NONBLOCK;
 	}
 	else
 	{
@@ -685,79 +964,47 @@ if (result==TRUE)
 	}
 }
 
-
 return(result);
 }
 
 
 
-
-
-int OpenUDPSock(int Port)
+int STREAMConnect(STREAM *S, const char *URL, int Flags)
 {
-	int result;
-	struct sockaddr_in addr;
-	int fd;
-
-	addr.sin_family=AF_INET;
-//	addr.sin_addr.s_addr=Interface;
-	addr.sin_addr.s_addr=INADDR_ANY;
-	addr.sin_port=htons(Port);
-
-	fd=socket(AF_INET, SOCK_DGRAM,0);
-	result=bind(fd,(struct sockaddr *) &addr, sizeof(addr));
-        if (result !=0)
-        {
-		close(fd);
-		return(-1);
-        }
-   return(fd);
-}
+ListNode *Curr;
+int result=FALSE, fd;
+unsigned int Port=0, TTL=0, ToS=0;
+char *Proto=NULL, *Host=NULL, *Token=NULL, *Path=NULL;
 
 
-int STREAMSendDgram(STREAM *S, const char *Host, int Port, char *Bytes, int len)
+ParseURL(URL, &Proto, &Host, &Token,NULL, NULL,&Path,NULL);
+S->Path=FormatStr(S->Path,URL);
+Port=strtoul(Token,0,10);
+
+STREAMSetFlushType(S,FLUSH_LINE,0,0);
+if (strcasecmp(Proto,"unix")==0) result=STREAMConnectUnixSocket(S, Host, SOCK_STREAM);
+if (strcasecmp(Proto,"unixdgm")==0) result=STREAMConnectUnixSocket(S, Host, SOCK_DGRAM);
+else if ((strcasecmp(Proto,"udp")==0) || (strcasecmp(Proto,"bcast")==0)) 
 {
-struct sockaddr_in sa;
-socklen_t salen;
-struct hostent *hostdata;
-
-sa.sin_port=htons(Port);
-sa.sin_family=AF_INET;
-inet_aton(Host,& sa.sin_addr);
-salen=sizeof(sa);
-
-if (IsIP4Address(Host))
-{
-   inet_aton(Host, (struct in_addr *) &sa.sin_addr);
+	fd=UDPOpen("", 0, Flags);
+	if (fd > -1) 
+	{
+		S->in_fd=fd;
+		S->out_fd=fd;
+		S->Type=STREAM_TYPE_UDP;
+		result=IPReconnect(fd,Host,Port,0);
+		if (strcasecmp(Proto,"bcast")==0) SockSetOptions(fd, SOCK_BROADCAST, 0);
+	}
 }
-else 
-{ 
-   hostdata=gethostbyname(Host);
-   if (!hostdata) 
-   {
-     return(-1);
-   }
-sa.sin_addr=*(struct in_addr *) *hostdata->h_addr_list;
+else if (strcasecmp(Proto,"ssl")==0) result=STREAMTCPConnect(S, Host, Port, TTL, ToS, Flags | CONNECT_SSL);
+else result=STREAMTCPConnect(S, Host, Port, TTL, ToS, Flags);
+
+DestroyString(Token);
+DestroyString(Proto);
+DestroyString(Host);
+DestroyString(Path);
+
+return(result);
 }
 
 
-return(sendto(S->out_fd,Bytes,len,0,(struct sockaddr *) &sa,salen));
-}
-
-
-
-STREAM *STREAMOpenUDP(int Port,int NonBlock)
-{
-int fd;
-STREAM *Stream;
-
-fd=OpenUDPSock(Port);
-if (fd <0) return(NULL);
-Stream=STREAMFromFD(fd);
-Stream->Path=FormatStr(Stream->Path,"udp::%d",Port);
-Stream->Type=STREAM_TYPE_UDP;
-return(Stream);
-}
-
-
-
